@@ -662,11 +662,6 @@ void
 parse_shared_options (ArgParser& ap)
 {
   int i;
-  float f;
-  if (ap.parse_opt ("--strength", f))
-    {
-      Params::water_delta = f / 1000;
-    }
   if (ap.parse_opt ("--short", i))
     {
       Params::payload_size = i;
@@ -727,6 +722,7 @@ parse_add_options (ArgParser& ap)
 {
   string s;
   int i;
+  float f;
 
   ap.parse_opt ("--set-input-label", Params::input_label);
   ap.parse_opt ("--set-output-label", Params::output_label);
@@ -807,6 +803,10 @@ parse_add_options (ArgParser& ap)
       error ("audiowmark: using rf64 as input format has no effect\n");
       exit (1);
     }
+  if (ap.parse_opt ("--strength", f))
+    {
+      Params::water_delta = f / 1000;
+    }
 }
 
 void
@@ -814,6 +814,7 @@ parse_get_options (ArgParser& ap)
 {
   string s;
   float f;
+  int i;
 
   ap.parse_opt ("--test-cut", Params::test_cut);
   ap.parse_opt ("--test-truncate", Params::test_truncate);
@@ -864,6 +865,19 @@ parse_get_options (ArgParser& ap)
         }
       Params::get_chunk_size = f;
     }
+  if (ap.parse_opt ("--sync-threshold", f))
+    {
+      Params::sync_threshold2 = f;
+    }
+  if (ap.parse_opt ("--n-best", i))
+    {
+      if (i < 0)
+        {
+          error ("audiowmark: --n-best should not be a negative number\n");
+          exit (1);
+        }
+      Params::get_n_best = i;
+    }
 }
 
 template <class ... Args>
